using Bulldozer.MovementStrategy;
using Bulldozer.DrawingObjects;

namespace Bulldozer
{
    public partial class FormBulldozer : Form
    {
        /// <summary>
        /// -   
        /// </summary>
        private DrawingBulldozer? _drawingBulldozer;

        /// <summary>
        ///  
        /// </summary>
        private AbstractStrategy? _abstractStrategy;

        /// <summary>
        ///  
        /// </summary>
        public DrawingBulldozer? SelectedBulldozer { get; private set; }

        /// <summary>
        ///  
        /// </summary>
        public FormBulldozer()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedBulldozer = null;
        }

        /// <summary>
        ///   
        /// </summary>
        private void Draw()
        {
            if (_drawingBulldozer == null)
            {
                return;
            }
            Bitmap bmp = new(pictureBoxBulldozer.Width, pictureBoxBulldozer.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingBulldozer.DrawBulldozer(gr);
            pictureBoxBulldozer.Image = bmp;
        }

        /// <summary>
        ///    "   "
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingBulldozer == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawingBulldozer.MoveBulldozer(DirectionType.Up);
                    break;
                case "buttonDown":
                    _drawingBulldozer.MoveBulldozer(DirectionType.Down);
                    break;
                case "buttonLeft":
                    _drawingBulldozer.MoveBulldozer(DirectionType.Left);
                    break;
                case "buttonRight":
                    _drawingBulldozer.MoveBulldozer(DirectionType.Right);
                    break;
            }
            Draw();
        }

        /// <summary>
        ///    ""
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCreateUpgradedBulldozer_Click(object sender, EventArgs e)
        {
            Random random = new Random();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); //  
            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            Color dopColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)); // .   EntityUpgradedBulldozer
            ColorDialog colorDialog = new();
            
            if (colorDialog.ShowDialog() == DialogResult.OK)
                bodyColor = colorDialog.Color;
            if (colorDialog.ShowDialog() == DialogResult.OK)
                dopColor = colorDialog.Color;
            _drawingBulldozer = new DrawingBulldozerUpgraded(
                random.Next(100, 300),   // 
                random.Next(1000, 3000), // 
                bodyColor, additionalColor, dopColor,
                Convert.ToBoolean(random.Next(2)),
                Convert.ToBoolean(random.Next(2)),
                pictureBoxBulldozer.Width,
                pictureBoxBulldozer.Height
            );

            _drawingBulldozer.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();

        }

        private void buttonCreateBulldozer_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color bodyColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
            ColorDialog colorDialog = new ColorDialog();
            

            if (colorDialog.ShowDialog() == DialogResult.OK)
            {
                bodyColor = colorDialog.Color;
            }

            Color additionalColor = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));

            _drawingBulldozer = new DrawingBulldozer(random.Next(100, 300),   // 
                random.Next(1000, 3000), // 
                bodyColor, additionalColor,
                pictureBoxBulldozer.Width,
                pictureBoxBulldozer.Height);

            _drawingBulldozer.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }

        private void buttonStep_Click(object sender, EventArgs e)
        {
            if (_drawingBulldozer == null)
            {
                return;
            }
            if (comboBoxStrategy.Enabled)
            {
                _abstractStrategy = comboBoxStrategy.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawingObjectBulldozer(_drawingBulldozer), pictureBoxBulldozer.Width, pictureBoxBulldozer.Height);
                comboBoxStrategy.Enabled = false;
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxStrategy.Enabled = true;
                _abstractStrategy = null;
            }
        }

        /// <summary>
        ///  
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void buttonSelectBulldozer_Click(object sender, EventArgs e)
        {
            SelectedBulldozer = _drawingBulldozer;
            DialogResult = DialogResult.OK;
        }
    }
}