﻿namespace Bulldozer.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T?[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _places = new T?[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="bulldozer">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T bulldozer)
        {
            if (_places[Count - 1] != null)
                return -1;
            return Insert(bulldozer, 0);

        }

        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="bulldozer">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T bulldozer, int position)
        {
            if (!(position >= 0 && position < Count))
                return -1;
            if (_places[position] != null)
            {
                int ind = position;
                while (ind < Count && _places[ind] != null)
                    ind++;
                if (ind == Count)
                    return -1;
                for (int i = ind - 1; i >= position; i--)
                    _places[i + 1] = _places[i];
            }
            _places[position] = bulldozer;
            return position;

        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (!(position >= 0 && position < Count) || _places[position] == null)
                return false;
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? Get(int position)
        {
            if (!(position >= 0 && position < Count))
                return null;
            return _places[position];
        }
    }
}